@@ -1,4 +1,4 @@
-/* $Id: configure.c,v 4.2 1998/04/17 07:48:54 bert Exp $
+/* $Id: configure.c,v 4.3 1998/09/09 00:22:16 dick Exp $
  *
  * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-98 by
  *
@@ -132,6 +132,7 @@ static int Config_create_autoShield(int widget_desc, int *height);
 static int Config_create_sparkProb(int widget_desc, int *height);
 static int Config_create_shotSize(int widget_desc, int *height);
 static int Config_create_teamShotSize(int widget_desc, int *height);
+static int Config_create_showNastyShots(int widget_desc, int *height);
 static int Config_create_hudColor(int widget_desc, int *height);
 static int Config_create_hudLockColor(int widget_desc, int *height);
 static int Config_create_wallColor(int widget_desc, int *height);
@@ -257,6 +258,7 @@ static int		(*config_creator[])(int widget_desc, int *height) = {
     Config_create_markingLights,
     Config_create_toggleShield,
     Config_create_autoShield,
+	Config_create_showNastyShots,
     Config_create_shotSize,
     Config_create_teamShotSize,
     Config_create_hudColor,
@@ -873,6 +875,13 @@ static int Config_create_teamShotSize(int widget_desc, int *height)
 			   NULL, NULL);
 }
 
+static int Config_create_showNastyShots(int widget_desc, int *height)
+{
+    return Config_create_bool(widget_desc, height, "nastyShots",
+                              (showNastyShots) ? true : false,
+                              Config_update_bool, &showNastyShots);
+}
+
 static int Config_create_hudColor(int widget_desc, int *height)
 {
     return Config_create_int(widget_desc, height,
@@ -1212,6 +1221,13 @@ static int Config_update_sparkProb(int widget_desc, void *data, DFLOAT *val)
     return 0;
 }
 
+static int Config_update_showNastyShots(int widget_desc, void *data, bool *val)
+{
+    Set_toggle_shield(*val != false);
+    return 0;
+}
+
+
 static int Config_update_charsPerSecond(int widget_desc, void *data, int *val)
 {
     charsPerTick = (DFLOAT)charsPerSecond / FPS;
@@ -1463,6 +1479,7 @@ static int Config_save(int widget_desc, void *button_str, const char **strptr)
     Config_save_float(fp, "sparkProb", spark_prob);
     Config_save_int(fp, "shotSize", shot_size);
     Config_save_int(fp, "teamShotSize", teamshot_size);
+	Config_save_bool(fp, "showNastyShots", showNastyShots);
     Config_save_int(fp, "hudColor", hudColor);
     Config_save_int(fp, "hudLockColor", hudLockColor);
     Config_save_int(fp, "wallColor", wallColor);
