@@ -14,7 +14,7 @@
  *
  * This software is provided "as is" without any express or implied warranty.
  *
- * RCS:      $Id: socklib.c,v 4.1 1998/04/18 12:53:03 bert Exp $
+ * RCS:      $Id: socklib.c,v 4.5 1998/09/18 15:38:02 bert Exp $
  *
  * Revision 1.1.1.1  1992/05/11  12:32:34  bjoerns
  * XPilot v1.0
@@ -30,7 +30,7 @@
 
 #ifndef lint
 static char sourceid[] =
-    "@(#)$Id: socklib.c,v 4.1 1998/04/18 12:53:03 bert Exp $";
+    "@(#)$Id: socklib.c,v 4.5 1998/09/18 15:38:02 bert Exp $";
 #endif
 
 #ifdef TERMNET
@@ -1813,23 +1813,25 @@ int DgramSendRec(int fd, char *host, int port, char *sbuf,
 	(void) alarm(sl_timeout_s, DgramInthandler);
 #endif
 	retval = DgramReceive(fd, host, rbuf, rbuf_size);
-	if (retval == -1)
-	    if (errno == EINTR || sl_errno == SL_EWRONGHOST)
+	if (retval == -1) {
+	    if (errno == EINTR || sl_errno == SL_EWRONGHOST) {
 		/* We have a timeout or a message from wrong host */
-		if (--retry)
+		if (--retry) {
 		    continue;	/* Try one more time */
-		else
-		{
+		}
+		else {
 		    sl_errno = SL_ENORESP;
 		    break;	/* Unable to get response */
 		}
-	    else
-	    {
+	    }
+	    else {
 		sl_errno = SL_ERECEIVE;
 		break;		/* Unable to receive response */
 	    }
-	else
+	}
+	else {
 	    break;		/* Datagram from <host> arrived */
+	}
     }
 #ifndef	_WINDOWS
     (void) alarm(0);
@@ -2033,13 +2035,13 @@ void DgramClose(int fd)
 void GetLocalHostName(char *name, unsigned size)
 {
     struct hostent	*he = NULL;
-    struct hostent 	 *xpilot_he = NULL;
+    struct hostent 	*xpilot_he = NULL;
 #ifndef	_WINDOWS
-    struct hostent	  tmp;
+    struct hostent	tmp;
 #endif
     int			xpilot_len;
-    char		*alias, *dot;
-    char		xpilot_hostname[MAXHOSTNAMELEN];
+    char		*alias, *dot; 
+    char		xpilot_hostname[MAXHOSTNAMELEN]; /* unused */
     static const char	xpilot[] = "xpilot";
 #ifdef VMS
     char                vms_inethost[MAXHOSTNAMELEN]   = "UCX$INET_HOST";
@@ -2049,30 +2051,19 @@ void GetLocalHostName(char *name, unsigned size)
     int                 namelen;
 #endif
 
-    xpilot_len = strlen(xpilot);
-
-#ifndef	_WINDOWS	/* this takes FOREVER! zzzz */
-    /* Make a wild guess that a "xpilot" hostname or alias is in this domain */
-    if ((xpilot_he = gethostbyname(xpilot)) != NULL) {
-	strcpy(xpilot_hostname, xpilot_he->h_name);	/* copy data to buffer */
-	tmp = *xpilot_he;
-	xpilot_he = &tmp;
-    }
-#endif
-
     gethostname(name, size);
     if ((he = gethostbyname(name)) == NULL) {
 	return;
     }
     strncpy(name, he->h_name, size);
     name[size - 1] = '\0';
+
     /*
      * If there are no dots in the name then we don't have the FQDN,
      * and if the address is of the normal Internet type
      * then we try to get the FQDN via the backdoor of the IP address.
      * Let's hope it works :)
      */
-
     if (strchr(he->h_name, '.') == NULL
 	&& he->h_addrtype == AF_INET
 	&& he->h_length == 4) {
@@ -2084,7 +2075,15 @@ void GetLocalHostName(char *name, unsigned size)
 	    name[size - 1] = '\0';
 	}
 	else {
-#if !defined(VMS)
+#if defined(VMS)
+            vms_trnlnm(vms_inethost, namelen, vms_host);
+            vms_trnlnm(vms_inetdomain, namelen, vms_domain);
+            strcpy(name, vms_host);
+            strcat(name, ".");
+            strcat(name, vms_domain);
+            return;
+#else
+	    /* Let's try to find the domain from /etc/resolv.conf. */
 	    FILE *fp = fopen("/etc/resolv.conf", "r");
 	    if (fp) {
 		char *s, buf[256];
@@ -2099,55 +2098,59 @@ void GetLocalHostName(char *name, unsigned size)
 		}
 		fclose(fp);
 	    }
-#else
-            vms_trnlnm(vms_inethost, namelen, vms_host);
-            vms_trnlnm(vms_inetdomain, namelen, vms_domain);
-            strcpy(name, vms_host);
-            strcat(name, ".");
-            strcat(name, vms_domain);
 #endif
+	}
+	/* make sure this is a valid FQDN. */
+	if ((he = gethostbyname(name)) == NULL) {
+	    gethostname(name, size);
 	    return;
 	}
     }
 
+#ifndef	_WINDOWS	/* the lookup of xpilot can take FOREVER! zzzz...  */
+
+    /* if name starts with "xpilot" then we're done. */
+    xpilot_len = strlen(xpilot);
+    if (!strncmp(name, xpilot, xpilot_len)) {
+	return;
+    }
+
+    /* copy hostent data to buffer */
+    tmp = *he;
+    he = &tmp;
+
+    /* Make a wild guess that a "xpilot" hostname or alias is in this domain */
+    strcpy(xpilot_hostname, xpilot);
+    if ((dot = strchr(name, '.')) != NULL) {
+	if (strlen(xpilot_hostname) + strlen(dot) < sizeof(xpilot_hostname)) {
+	    strcat(xpilot_hostname, dot);
+	}
+    }
+    if ((xpilot_he = gethostbyname(xpilot_hostname)) == NULL) {
+	return;
+    }
+
     /*
      * If a "xpilot" host is found compare if it's this one.
-     * and if so, make the local name as "xpilot.*"
+     * and if so, make the local name as "xpilot.domain"
      */
-    if (xpilot_he != NULL) {               /* host xpilot was found */
-	if (strcmp(he->h_name, xpilot_hostname) == 0) {
-	   /*
-	    * Identical official names. Can they be different hosts after this?
-	    * Find out the name which starts with "xpilot" and use it:
-	    */
-	    xpilot_he = gethostbyname(xpilot); /* read again the aliases info */
-	    if (xpilot_he == NULL)       /* shouldn't happen */
-		return;
-
-	    if (strncmp(xpilot, xpilot_he->h_name, xpilot_len) != 0) {
-		/*
-		 * the official hostname doesn't begin "xpilot"
-		 * so we'll find the alias:
-		 */
-		int i;
-		for (i = 0; xpilot_he->h_aliases[i] != NULL; i++) {
-		    alias = xpilot_he->h_aliases[i];
-		    if (!strncmp(xpilot, alias, xpilot_len)) {
-			strcpy(xpilot_hostname, alias);
-			if (!strchr(alias, '.') && (dot = strchr(name, '.'))) {
-			    strcat(xpilot_hostname + strlen(xpilot_hostname), dot);
-			}
-			strncpy(name, xpilot_hostname, size);
-			return;
-		    }
-		}
-	    } else {
-		strncpy(name, xpilot_he->h_name, size);
+    if (!strcmp(he->h_name, xpilot_he->h_name)) {
+       /*
+	* Identical official names. Can they be different hosts after this?
+	* The official hostname doesn't begin "xpilot" so we'll find the alias:
+	*/
+	int i;
+	for (i = 0; xpilot_he->h_aliases[i] != NULL; i++) {
+	    alias = xpilot_he->h_aliases[i];
+	    if (!strncmp(xpilot, alias, xpilot_len)) {
+		strncpy(name, xpilot_hostname, size);
+		name[size - 1] = '\0';
 		return;
 	    }
 	}
-	/* NOT REATCHED */
     }
+
+#endif
 } /* GetLocalHostName */
 
 
